package solutions.bitbadger.documents.core.tests.java;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import solutions.bitbadger.documents.DocumentException;
import solutions.bitbadger.documents.Field;
import solutions.bitbadger.documents.query.FindQuery;
import solutions.bitbadger.documents.core.tests.ForceDialect;

import java.util.List;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static solutions.bitbadger.documents.core.tests.TypesKt.TEST_TABLE;

/**
 * Unit tests for the `Find` object
 */
@DisplayName("Core | Java | Query | FindQuery")
final public class FindQueryTest {

    /**
     * Clear the connection string (resets Dialect)
     */
    @AfterEach
    public void cleanUp() {
        ForceDialect.none();
    }

    @Test
    @DisplayName("all generates correctly")
    public void all() {
        assertEquals(String.format("SELECT data FROM %s", TEST_TABLE), FindQuery.all(TEST_TABLE),
                "Find query not constructed correctly");
    }

    @Test
    @DisplayName("byId generates correctly | PostgreSQL")
    public void byIdPostgres() throws DocumentException {
        ForceDialect.postgres();
        assertEquals(String.format("SELECT data FROM %s WHERE data->>'id' = :id", TEST_TABLE),
                FindQuery.byId(TEST_TABLE), "Find query not constructed correctly");
    }

    @Test
    @DisplayName("byId generates correctly | SQLite")
    public void byIdSQLite() throws DocumentException {
        ForceDialect.sqlite();
        assertEquals(String.format("SELECT data FROM %s WHERE data->>'id' = :id", TEST_TABLE),
                FindQuery.byId(TEST_TABLE), "Find query not constructed correctly");
    }

    @Test
    @DisplayName("byFields generates correctly | PostgreSQL")
    public void byFieldsPostgres() throws DocumentException {
        ForceDialect.postgres();
        assertEquals(
                String.format("SELECT data FROM %s WHERE data->>'a' = :b AND (data->>'c')::numeric < :d", TEST_TABLE),
                FindQuery.byFields(TEST_TABLE, List.of(Field.equal("a", "", ":b"), Field.less("c", 14, ":d"))),
                "Find query not constructed correctly");
    }

    @Test
    @DisplayName("byFields generates correctly | SQLite")
    public void byFieldsSQLite() throws DocumentException {
        ForceDialect.sqlite();
        assertEquals(String.format("SELECT data FROM %s WHERE data->>'a' = :b AND data->>'c' < :d", TEST_TABLE),
                FindQuery.byFields(TEST_TABLE, List.of(Field.equal("a", "", ":b"), Field.less("c", 14, ":d"))),
                "Find query not constructed correctly");
    }

    @Test
    @DisplayName("byContains generates correctly | PostgreSQL")
    public void byContainsPostgres() throws DocumentException {
        ForceDialect.postgres();
        assertEquals(String.format("SELECT data FROM %s WHERE data @> :criteria", TEST_TABLE),
                FindQuery.byContains(TEST_TABLE), "Find query not constructed correctly");
    }

    @Test
    @DisplayName("byContains fails | SQLite")
    public void byContainsSQLite() {
        ForceDialect.sqlite();
        assertThrows(DocumentException.class, () -> FindQuery.byContains(TEST_TABLE));
    }

    @Test
    @DisplayName("byJsonPath generates correctly | PostgreSQL")
    public void byJsonPathPostgres() throws DocumentException {
        ForceDialect.postgres();
        assertEquals(String.format("SELECT data FROM %s WHERE jsonb_path_exists(data, :path::jsonpath)", TEST_TABLE),
                FindQuery.byJsonPath(TEST_TABLE), "Find query not constructed correctly");
    }

    @Test
    @DisplayName("byJsonPath fails | SQLite")
    public void byJsonPathSQLite() {
        ForceDialect.sqlite();
        assertThrows(DocumentException.class, () -> FindQuery.byJsonPath(TEST_TABLE));
    }
}
