package solutions.bitbadger.documents.core.tests.java.integration;

import solutions.bitbadger.documents.DocumentException;
import solutions.bitbadger.documents.DocumentIndex;
import solutions.bitbadger.documents.core.tests.integration.ThrowawayDatabase;

import java.util.List;

import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertTrue;
import static solutions.bitbadger.documents.core.tests.TypesKt.TEST_TABLE;
import static solutions.bitbadger.documents.java.extensions.ConnExt.*;

final public class DefinitionFunctions {

    public static void ensuresATable(ThrowawayDatabase db) throws DocumentException {
        assertFalse(db.dbObjectExists("ensured"), "The 'ensured' table should not exist");
        assertFalse(db.dbObjectExists("idx_ensured_key"), "The PK index for the 'ensured' table should not exist");
        ensureTable(db.getConn(), "ensured");
        assertTrue(db.dbObjectExists("ensured"), "The 'ensured' table should exist");
        assertTrue(db.dbObjectExists("idx_ensured_key"), "The PK index for the 'ensured' table should now exist");
    }

    public static void ensuresAFieldIndex(ThrowawayDatabase db) throws DocumentException {
        assertFalse(db.dbObjectExists(String.format("idx_%s_test", TEST_TABLE)), "The test index should not exist");
        ensureFieldIndex(db.getConn(), TEST_TABLE, "test", List.of("id", "category"));
        assertTrue(db.dbObjectExists(String.format("idx_%s_test", TEST_TABLE)), "The test index should now exist");
    }

    public static void ensureDocumentIndexFull(ThrowawayDatabase db) throws DocumentException {
        assertFalse(db.dbObjectExists("doc_table"), "The 'doc_table' table should not exist");
        ensureTable(db.getConn(), "doc_table");
        assertTrue(db.dbObjectExists("doc_table"), "The 'doc_table' table should exist");
        assertFalse(db.dbObjectExists("idx_doc_table_document"), "The document index should not exist");
        ensureDocumentIndex(db.getConn(), "doc_table", DocumentIndex.FULL);
        assertTrue(db.dbObjectExists("idx_doc_table_document"), "The document index should exist");
    }

    public static void ensureDocumentIndexOptimized(ThrowawayDatabase db) throws DocumentException {
        assertFalse(db.dbObjectExists("doc_table"), "The 'doc_table' table should not exist");
        ensureTable(db.getConn(), "doc_table");
        assertTrue(db.dbObjectExists("doc_table"), "The 'doc_table' table should exist");
        assertFalse(db.dbObjectExists("idx_doc_table_document"), "The document index should not exist");
        ensureDocumentIndex(db.getConn(), "doc_table", DocumentIndex.OPTIMIZED);
        assertTrue(db.dbObjectExists("idx_doc_table_document"), "The document index should exist");
    }
}
