package solutions.bitbadger.documents.core.tests.java.integration;

import solutions.bitbadger.documents.DocumentException;
import solutions.bitbadger.documents.Field;
import solutions.bitbadger.documents.core.tests.integration.ThrowawayDatabase;

import java.util.List;
import java.util.Map;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static solutions.bitbadger.documents.core.tests.TypesKt.TEST_TABLE;
import static solutions.bitbadger.documents.java.extensions.ConnExt.*;

final public class DeleteFunctions {

    public static void byIdMatch(ThrowawayDatabase db) throws DocumentException {
        JsonDocument.load(db);
        assertEquals(5L, countAll(db.getConn(), TEST_TABLE), "There should be 5 documents in the table");
        deleteById(db.getConn(), TEST_TABLE, "four");
        assertEquals(4L, countAll(db.getConn(), TEST_TABLE), "There should now be 4 documents in the table");
    }

    public static void byIdNoMatch(ThrowawayDatabase db) throws DocumentException {
        JsonDocument.load(db);
        assertEquals(5L, countAll(db.getConn(), TEST_TABLE), "There should be 5 documents in the table");
        deleteById(db.getConn(), TEST_TABLE, "negative four");
        assertEquals(5L, countAll(db.getConn(), TEST_TABLE), "There should still be 5 documents in the table");
    }

    public static void byFieldsMatch(ThrowawayDatabase db) throws DocumentException {
        JsonDocument.load(db);
        assertEquals(5L, countAll(db.getConn(), TEST_TABLE), "There should be 5 documents in the table");
        deleteByFields( db.getConn(), TEST_TABLE, List.of(Field.notEqual("value", "purple")));
        assertEquals(2L, countAll(db.getConn(), TEST_TABLE), "There should now be 2 documents in the table");
    }

    public static void byFieldsNoMatch(ThrowawayDatabase db) throws DocumentException {
        JsonDocument.load(db);
        assertEquals(5L, countAll(db.getConn(), TEST_TABLE), "There should be 5 documents in the table");
        deleteByFields(db.getConn(), TEST_TABLE, List.of(Field.equal("value", "crimson")));
        assertEquals(5L, countAll(db.getConn(), TEST_TABLE), "There should still be 5 documents in the table");
    }

    public static void byContainsMatch(ThrowawayDatabase db) throws DocumentException {
        JsonDocument.load(db);
        assertEquals(5L, countAll(db.getConn(), TEST_TABLE), "There should be 5 documents in the table");
        deleteByContains(db.getConn(), TEST_TABLE, Map.of("value", "purple"));
        assertEquals(3L, countAll(db.getConn(), TEST_TABLE), "There should now be 3 documents in the table");
    }

    public static void byContainsNoMatch(ThrowawayDatabase db) throws DocumentException {
        JsonDocument.load(db);
        assertEquals(5L, countAll(db.getConn(), TEST_TABLE), "There should be 5 documents in the table");
        deleteByContains(db.getConn(), TEST_TABLE, Map.of("target", "acquired"));
        assertEquals(5L, countAll(db.getConn(), TEST_TABLE), "There should still be 5 documents in the table");
    }

    public static void byJsonPathMatch(ThrowawayDatabase db) throws DocumentException {
        JsonDocument.load(db);
        assertEquals(5L, countAll(db.getConn(), TEST_TABLE), "There should be 5 documents in the table");
        deleteByJsonPath(db.getConn(), TEST_TABLE, "$.value ? (@ == \"purple\")");
        assertEquals(3L, countAll(db.getConn(), TEST_TABLE), "There should now be 3 documents in the table");
    }

    public static void byJsonPathNoMatch(ThrowawayDatabase db) throws DocumentException {
        JsonDocument.load(db);
        assertEquals(5L, countAll(db.getConn(), TEST_TABLE), "There should be 5 documents in the table");
        deleteByJsonPath(db.getConn(), TEST_TABLE, "$.numValue ? (@ > 100)");
        assertEquals(5L, countAll(db.getConn(), TEST_TABLE), "There should still be 5 documents in the table");
    }
}
