package solutions.bitbadger.documents.core.tests.java.integration;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import solutions.bitbadger.documents.DocumentException;
import solutions.bitbadger.documents.core.tests.integration.SQLiteDB;

import static org.junit.jupiter.api.Assertions.assertThrows;

/**
 * SQLite integration tests for the `Delete` object / `deleteBy*` connection extension functions
 */
@DisplayName("Core | Java | SQLite: Delete")
final public class SQLiteDeleteIT {

    @Test
    @DisplayName("byId deletes a matching ID")
    public void byIdMatch() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            DeleteFunctions.byIdMatch(db);
        }
    }

    @Test
    @DisplayName("byId succeeds when no ID matches")
    public void byIdNoMatch() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            DeleteFunctions.byIdNoMatch(db);
        }
    }

    @Test
    @DisplayName("byFields deletes matching documents")
    public void byFieldsMatch() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            DeleteFunctions.byFieldsMatch(db);
        }
    }

    @Test
    @DisplayName("byFields succeeds when no documents match")
    public void byFieldsNoMatch() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            DeleteFunctions.byFieldsNoMatch(db);
        }
    }

    @Test
    @DisplayName("byContains fails")
    public void byContainsMatch() {
        try (SQLiteDB db = new SQLiteDB()) {
            assertThrows(DocumentException.class, () -> DeleteFunctions.byContainsMatch(db));
        }
    }

    @Test
    @DisplayName("byJsonPath fails")
    public void byJsonPathMatch() {
        try (SQLiteDB db = new SQLiteDB()) {
            assertThrows(DocumentException.class, () -> DeleteFunctions.byJsonPathMatch(db));
        }
    }
}
