package solutions.bitbadger.documents.core.tests.java.integration;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import solutions.bitbadger.documents.DocumentException;
import solutions.bitbadger.documents.core.tests.integration.SQLiteDB;

/**
 * SQLite integration tests for the `Document` object / `insert`, `save`, `update` connection extension functions
 */
@DisplayName("Core | Java | SQLite: Document")
final public class SQLiteDocumentIT {
    @Test
    @DisplayName("insert works with default values")
    public void insertDefault() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            DocumentFunctions.insertDefault(db);
        }
    }

    @Test
    @DisplayName("insert fails with duplicate key")
    public void insertDupe() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            DocumentFunctions.insertDupe(db);
        }
    }

    @Test
    @DisplayName("insert succeeds with numeric auto IDs")
    public void insertNumAutoId() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            DocumentFunctions.insertNumAutoId(db);
        }
    }

    @Test
    @DisplayName("insert succeeds with UUID auto ID")
    public void insertUUIDAutoId() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            DocumentFunctions.insertUUIDAutoId(db);
        }
    }

    @Test
    @DisplayName("insert succeeds with random string auto ID")
    public void insertStringAutoId() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            DocumentFunctions.insertStringAutoId(db);
        }
    }

    @Test
    @DisplayName("save updates an existing document")
    public void saveMatch() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            DocumentFunctions.saveMatch(db);
        }
    }

    @Test
    @DisplayName("save inserts a new document")
    public void saveNoMatch() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            DocumentFunctions.saveNoMatch(db);
        }
    }

    @Test
    @DisplayName("update replaces an existing document")
    public void updateMatch() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            DocumentFunctions.updateMatch(db);
        }
    }

    @Test
    @DisplayName("update succeeds when no document exists")
    public void updateNoMatch() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            DocumentFunctions.updateNoMatch(db);
        }
    }
}
