package solutions.bitbadger.documents.core.tests.java.integration;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import solutions.bitbadger.documents.DocumentException;
import solutions.bitbadger.documents.core.tests.integration.SQLiteDB;

import static org.junit.jupiter.api.Assertions.assertThrows;

/**
 * SQLite integration tests for the `Patch` object / `patchBy*` connection extension functions
 */
@DisplayName("Core | Java | SQLite: Patch")
final public class SQLitePatchIT {

    @Test
    @DisplayName("byId patches an existing document")
    public void byIdMatch() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            PatchFunctions.byIdMatch(db);
        }
    }

    @Test
    @DisplayName("byId succeeds for a non-existent document")
    public void byIdNoMatch() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            PatchFunctions.byIdNoMatch(db);
        }
    }

    @Test
    @DisplayName("byFields patches matching document")
    public void byFieldsMatch() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            PatchFunctions.byFieldsMatch(db);
        }
    }

    @Test
    @DisplayName("byFields succeeds when no documents match")
    public void byFieldsNoMatch() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            PatchFunctions.byFieldsNoMatch(db);
        }
    }

    @Test
    @DisplayName("byContains fails")
    public void byContainsMatch() {
        try (SQLiteDB db = new SQLiteDB()) {
            assertThrows(DocumentException.class, () -> ExistsFunctions.byContainsMatch(db));
        }
    }

    @Test
    @DisplayName("byJsonPath fails")
    public void byJsonPathMatch() {
        try (SQLiteDB db = new SQLiteDB()) {
            assertThrows(DocumentException.class, () -> ExistsFunctions.byJsonPathMatch(db));
        }
    }
}
