package solutions.bitbadger.documents.core.tests

import org.junit.jupiter.api.AfterEach
import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.assertThrows
import solutions.bitbadger.documents.DocumentException
import solutions.bitbadger.documents.Field
import solutions.bitbadger.documents.query.PatchQuery
import kotlin.test.assertEquals

/**
 * Unit tests for the `Patch` object
 */
@DisplayName("Core | Kotlin | Query | PatchQuery")
class PatchQueryTest {

    /**
     * Reset the dialect
     */
    @AfterEach
    fun cleanUp() {
        ForceDialect.none()
    }

    @Test
    @DisplayName("byId generates correctly | PostgreSQL")
    fun byIdPostgres() {
        ForceDialect.postgres()
        assertEquals(
            "UPDATE $TEST_TABLE SET data = data || :data WHERE data->>'id' = :id",
            PatchQuery.byId<String>(TEST_TABLE), "Patch query not constructed correctly"
        )
    }

    @Test
    @DisplayName("byId generates correctly | SQLite")
    fun byIdSQLite() {
        ForceDialect.sqlite()
        assertEquals(
            "UPDATE $TEST_TABLE SET data = json_patch(data, json(:data)) WHERE data->>'id' = :id",
            PatchQuery.byId<String>(TEST_TABLE), "Patch query not constructed correctly"
        )
    }

    @Test
    @DisplayName("byFields generates correctly | PostgreSQL")
    fun byFieldsPostgres() {
        ForceDialect.postgres()
        assertEquals(
            "UPDATE $TEST_TABLE SET data = data || :data WHERE data->>'z' = :y",
            PatchQuery.byFields(TEST_TABLE, listOf(Field.equal("z", "", ":y"))),
            "Patch query not constructed correctly"
        )
    }

    @Test
    @DisplayName("byFields generates correctly | SQLite")
    fun byFieldsSQLite() {
        ForceDialect.sqlite()
        assertEquals(
            "UPDATE $TEST_TABLE SET data = json_patch(data, json(:data)) WHERE data->>'z' = :y",
            PatchQuery.byFields(TEST_TABLE, listOf(Field.equal("z", "", ":y"))),
            "Patch query not constructed correctly"
        )
    }

    @Test
    @DisplayName("byContains generates correctly | PostgreSQL")
    fun byContainsPostgres() {
        ForceDialect.postgres()
        assertEquals(
            "UPDATE $TEST_TABLE SET data = data || :data WHERE data @> :criteria", PatchQuery.byContains(TEST_TABLE),
            "Patch query not constructed correctly"
        )
    }

    @Test
    @DisplayName("byContains fails | SQLite")
    fun byContainsSQLite() {
        ForceDialect.sqlite()
        assertThrows<DocumentException> { PatchQuery.byContains(TEST_TABLE) }
    }

    @Test
    @DisplayName("byJsonPath generates correctly | PostgreSQL")
    fun byJsonPathPostgres() {
        ForceDialect.postgres()
        assertEquals(
            "UPDATE $TEST_TABLE SET data = data || :data WHERE jsonb_path_exists(data, :path::jsonpath)",
            PatchQuery.byJsonPath(TEST_TABLE), "Patch query not constructed correctly"
        )
    }

    @Test
    @DisplayName("byJsonPath fails | SQLite")
    fun byJsonPathSQLite() {
        ForceDialect.sqlite()
        assertThrows<DocumentException> { PatchQuery.byJsonPath(TEST_TABLE) }
    }
}
