package solutions.bitbadger.documents.core.tests

import solutions.bitbadger.documents.core.tests.integration.ThrowawayDatabase
import solutions.bitbadger.documents.java.Document

/** The test table name to use for unit/integration tests */
const val TEST_TABLE = "test_table"

data class NumIdDocument(val key: Int, val text: String) {
    constructor() : this(0, "")
}

data class SubDocument(val foo: String, val bar: String) {
    constructor() : this("", "")
}

data class ArrayDocument(val id: String, val values: List<String>) {

    constructor() : this("", listOf())

    companion object {
        /** A set of documents used for integration tests */
        val testDocuments = listOf(
            ArrayDocument("first",  listOf("a", "b", "c")),
            ArrayDocument("second", listOf("c", "d", "e")),
            ArrayDocument("third",  listOf("x", "y", "z"))
        )
    }
}

data class JsonDocument(val id: String, val value: String = "", val numValue: Int = 0, val sub: SubDocument? = null) {

    constructor() : this("")

    companion object {
        /** Documents to use for testing */
        private val testDocuments = listOf(
            JsonDocument("one",   "FIRST!",   0, null),
            JsonDocument("two",   "another", 10, SubDocument("green", "blue")),
            JsonDocument("three", "",         4, null),
            JsonDocument("four",  "purple",  17, SubDocument("green", "red")),
            JsonDocument("five",  "purple",  18, null)
        )

        fun load(db: ThrowawayDatabase, tableName: String = TEST_TABLE) =
            testDocuments.forEach { Document.insert(tableName, it, db.conn) }

        /** Document ID `one` as a JSON string */
        val one = """{"id":"one","value":"FIRST!","numValue":0,"sub":null}"""

        /** Document ID `two` as a JSON string */
        val two = """{"id":"two","value":"another","numValue":10,"sub":{"foo":"green","bar":"blue"}}"""

        /** Document ID `three` as a JSON string */
        val three = """{"id":"three","value":"","numValue":4,"sub":null}"""

        /** Document ID `four` as a JSON string */
        val four = """{"id":"four","value":"purple","numValue":17,"sub":{"foo":"green","bar":"red"}}"""

        /** Document ID `five` as a JSON string */
        val five = """{"id":"five","value":"purple","numValue":18,"sub":null}"""
    }
}
