package solutions.bitbadger.documents.core.tests.integration

import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.assertThrows
import solutions.bitbadger.documents.DocumentException
import kotlin.test.Test

/**
 * SQLite integration tests for the `Json` object / `json*` connection extension functions
 */
@DisplayName("Core | Kotlin | SQLite: Json")
class SQLiteJsonIT {

    @Test
    @DisplayName("all retrieves all documents")
    fun allDefault() =
        SQLiteDB().use(JsonFunctions::allDefault)

    @Test
    @DisplayName("all succeeds with an empty table")
    fun allEmpty() =
        SQLiteDB().use(JsonFunctions::allEmpty)

    @Test
    @DisplayName("byId retrieves a document via a string ID")
    fun byIdString() =
        SQLiteDB().use(JsonFunctions::byIdString)

    @Test
    @DisplayName("byId retrieves a document via a numeric ID")
    fun byIdNumber() =
        SQLiteDB().use(JsonFunctions::byIdNumber)

    @Test
    @DisplayName("byId returns null when a matching ID is not found")
    fun byIdNotFound() =
        SQLiteDB().use(JsonFunctions::byIdNotFound)

    @Test
    @DisplayName("byFields retrieves matching documents")
    fun byFieldsMatch() =
        SQLiteDB().use(JsonFunctions::byFieldsMatch)

    @Test
    @DisplayName("byFields retrieves ordered matching documents")
    fun byFieldsMatchOrdered() =
        SQLiteDB().use(JsonFunctions::byFieldsMatchOrdered)

    @Test
    @DisplayName("byFields retrieves matching documents with a numeric IN clause")
    fun byFieldsMatchNumIn() =
        SQLiteDB().use(JsonFunctions::byFieldsMatchNumIn)

    @Test
    @DisplayName("byFields succeeds when no documents match")
    fun byFieldsNoMatch() =
        SQLiteDB().use(JsonFunctions::byFieldsNoMatch)

    @Test
    @DisplayName("byFields retrieves matching documents with an IN_ARRAY comparison")
    fun byFieldsMatchInArray() =
        SQLiteDB().use(JsonFunctions::byFieldsMatchInArray)

    @Test
    @DisplayName("byFields succeeds when no documents match an IN_ARRAY comparison")
    fun byFieldsNoMatchInArray() =
        SQLiteDB().use(JsonFunctions::byFieldsNoMatchInArray)

    @Test
    @DisplayName("byContains fails")
    fun byContainsFails() {
        assertThrows<DocumentException> { SQLiteDB().use(JsonFunctions::byContainsMatch) }
    }

    @Test
    @DisplayName("byJsonPath fails")
    fun byJsonPathFails() {
        assertThrows<DocumentException> { SQLiteDB().use(JsonFunctions::byJsonPathMatch) }
    }

    @Test
    @DisplayName("firstByFields retrieves a matching document")
    fun firstByFieldsMatchOne() =
        SQLiteDB().use(JsonFunctions::firstByFieldsMatchOne)

    @Test
    @DisplayName("firstByFields retrieves a matching document among many")
    fun firstByFieldsMatchMany() =
        SQLiteDB().use(JsonFunctions::firstByFieldsMatchMany)

    @Test
    @DisplayName("firstByFields retrieves a matching document among many (ordered)")
    fun firstByFieldsMatchOrdered() =
        SQLiteDB().use(JsonFunctions::firstByFieldsMatchOrdered)

    @Test
    @DisplayName("firstByFields returns null when no document matches")
    fun firstByFieldsNoMatch() =
        SQLiteDB().use(JsonFunctions::firstByFieldsNoMatch)

    @Test
    @DisplayName("firstByContains fails")
    fun firstByContainsFails() {
        assertThrows<DocumentException> { SQLiteDB().use(JsonFunctions::firstByContainsMatchOne) }
    }

    @Test
    @DisplayName("firstByJsonPath fails")
    fun firstByJsonPathFails() {
        assertThrows<DocumentException> { SQLiteDB().use(JsonFunctions::firstByJsonPathMatchOne) }
    }

    @Test
    @DisplayName("writeAll retrieves all documents")
    fun writeAllDefault() =
        SQLiteDB().use(JsonFunctions::writeAllDefault)

    @Test
    @DisplayName("writeAll succeeds with an empty table")
    fun writeAllEmpty() =
        SQLiteDB().use(JsonFunctions::writeAllEmpty)

    @Test
    @DisplayName("writeById retrieves a document via a string ID")
    fun writeByIdString() =
        SQLiteDB().use(JsonFunctions::writeByIdString)

    @Test
    @DisplayName("writeById retrieves a document via a numeric ID")
    fun writeByIdNumber() =
        SQLiteDB().use(JsonFunctions::writeByIdNumber)

    @Test
    @DisplayName("writeById returns null when a matching ID is not found")
    fun writeByIdNotFound() =
        SQLiteDB().use(JsonFunctions::writeByIdNotFound)

    @Test
    @DisplayName("writeByFields retrieves matching documents")
    fun writeByFieldsMatch() =
        SQLiteDB().use(JsonFunctions::writeByFieldsMatch)

    @Test
    @DisplayName("writeByFields retrieves ordered matching documents")
    fun writeByFieldsMatchOrdered() =
        SQLiteDB().use(JsonFunctions::writeByFieldsMatchOrdered)

    @Test
    @DisplayName("writeByFields retrieves matching documents with a numeric IN clause")
    fun writeByFieldsMatchNumIn() =
        SQLiteDB().use(JsonFunctions::writeByFieldsMatchNumIn)

    @Test
    @DisplayName("writeByFields succeeds when no documents match")
    fun writeByFieldsNoMatch() =
        SQLiteDB().use(JsonFunctions::writeByFieldsNoMatch)

    @Test
    @DisplayName("writeByFields retrieves matching documents with an IN_ARRAY comparison")
    fun writeByFieldsMatchInArray() =
        SQLiteDB().use(JsonFunctions::writeByFieldsMatchInArray)

    @Test
    @DisplayName("writeByFields succeeds when no documents match an IN_ARRAY comparison")
    fun writeByFieldsNoMatchInArray() =
        SQLiteDB().use(JsonFunctions::writeByFieldsNoMatchInArray)

    @Test
    @DisplayName("writeByContains fails")
    fun writeByContainsFails() {
        assertThrows<DocumentException> { SQLiteDB().use(JsonFunctions::writeByContainsMatch) }
    }

    @Test
    @DisplayName("writeByJsonPath fails")
    fun writeByJsonPathFails() {
        assertThrows<DocumentException> { SQLiteDB().use(JsonFunctions::writeByJsonPathMatch) }
    }

    @Test
    @DisplayName("writeFirstByFields retrieves a matching document")
    fun writeFirstByFieldsMatchOne() =
        SQLiteDB().use(JsonFunctions::writeFirstByFieldsMatchOne)

    @Test
    @DisplayName("writeFirstByFields retrieves a matching document among many")
    fun writeFirstByFieldsMatchMany() =
        SQLiteDB().use(JsonFunctions::writeFirstByFieldsMatchMany)

    @Test
    @DisplayName("writeFirstByFields retrieves a matching document among many (ordered)")
    fun writeFirstByFieldsMatchOrdered() =
        SQLiteDB().use(JsonFunctions::writeFirstByFieldsMatchOrdered)

    @Test
    @DisplayName("writeFirstByFields returns null when no document matches")
    fun writeFirstByFieldsNoMatch() =
        SQLiteDB().use(JsonFunctions::writeFirstByFieldsNoMatch)

    @Test
    @DisplayName("writeFirstByContains fails")
    fun writeFirstByContainsFails() {
        assertThrows<DocumentException> { SQLiteDB().use(JsonFunctions::writeFirstByContainsMatchOne) }
    }

    @Test
    @DisplayName("writeFirstByJsonPath fails")
    fun writeFirstByJsonPathFails() {
        assertThrows<DocumentException> { SQLiteDB().use(JsonFunctions::writeFirstByJsonPathMatchOne) }
    }
}
