package solutions.bitbadger.documents.groovy.tests

import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.Test
import solutions.bitbadger.documents.AutoId
import solutions.bitbadger.documents.Configuration
import solutions.bitbadger.documents.Dialect
import solutions.bitbadger.documents.DocumentException

import static org.junit.jupiter.api.Assertions.*

/**
 * Unit tests for the `Configuration` object
 */
@DisplayName('Groovy | Configuration')
class ConfigurationTest {

    @Test
    @DisplayName('Default ID field is "id"')
    void defaultIdField() {
        assertEquals 'id', Configuration.idField, 'Default ID field incorrect'
    }

    @Test
    @DisplayName('Default Auto ID strategy is DISABLED')
    void defaultAutoId() {
        assertEquals AutoId.DISABLED, Configuration.autoIdStrategy, 'Default Auto ID strategy should be DISABLED'
    }

    @Test
    @DisplayName('Default ID string length should be 16')
    void defaultIdStringLength() {
        assertEquals 16, Configuration.idStringLength, 'Default ID string length should be 16'
    }

    @Test
    @DisplayName('Dialect is derived from connection string')
    void dialectIsDerived() {
        try {
            assertThrows(DocumentException) { Configuration.dialect() }
            Configuration.connectionString = 'jdbc:postgresql:db'
            assertEquals Dialect.POSTGRESQL, Configuration.dialect()
        } finally {
            Configuration.connectionString = null
        }
    }
}
