package solutions.bitbadger.documents.groovy.tests

import org.junit.jupiter.api.AfterEach
import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.Test
import solutions.bitbadger.documents.DocumentException
import solutions.bitbadger.documents.Field
import solutions.bitbadger.documents.query.CountQuery

import static Types.TEST_TABLE
import static org.junit.jupiter.api.Assertions.*

/**
 * Unit tests for the `Count` object
 */
@DisplayName('Groovy | Query | CountQuery')
class CountQueryTest {

    /**
     * Clear the connection string (resets Dialect)
     */
    @AfterEach
    void cleanUp() {
        ForceDialect.none()
    }

    @Test
    @DisplayName('all generates correctly')
    void all() {
        assertEquals("SELECT COUNT(*) AS it FROM $TEST_TABLE".toString(), CountQuery.all(TEST_TABLE),
                'Count query not constructed correctly')
    }

    @Test
    @DisplayName('byFields generates correctly | PostgreSQL')
    void byFieldsPostgres() {
        ForceDialect.postgres()
        assertEquals("SELECT COUNT(*) AS it FROM $TEST_TABLE WHERE data->>'test' = :field0".toString(),
                CountQuery.byFields(TEST_TABLE, List.of(Field.equal('test', '', ':field0'))),
                'Count query not constructed correctly')
    }

    @Test
    @DisplayName('byFields generates correctly | SQLite')
    void byFieldsSQLite() {
        ForceDialect.sqlite()
        assertEquals("SELECT COUNT(*) AS it FROM $TEST_TABLE WHERE data->>'test' = :field0".toString(),
                CountQuery.byFields(TEST_TABLE, List.of(Field.equal('test', '', ':field0'))),
                'Count query not constructed correctly')
    }

    @Test
    @DisplayName('byContains generates correctly | PostgreSQL')
    void byContainsPostgres() {
        ForceDialect.postgres()
        assertEquals("SELECT COUNT(*) AS it FROM $TEST_TABLE WHERE data @> :criteria".toString(),
                CountQuery.byContains(TEST_TABLE), 'Count query not constructed correctly')
    }

    @Test
    @DisplayName('byContains fails | SQLite')
    void byContainsSQLite() {
        ForceDialect.sqlite()
        assertThrows(DocumentException) { CountQuery.byContains(TEST_TABLE) }
    }

    @Test
    @DisplayName('byJsonPath generates correctly | PostgreSQL')
    void byJsonPathPostgres() {
        ForceDialect.postgres()
        assertEquals("SELECT COUNT(*) AS it FROM $TEST_TABLE WHERE jsonb_path_exists(data, :path::jsonpath)".toString(),
                CountQuery.byJsonPath(TEST_TABLE), 'Count query not constructed correctly')
    }

    @Test
    @DisplayName('byJsonPath fails | SQLite')
    void byJsonPathSQLite() {
        ForceDialect.sqlite()
        assertThrows(DocumentException) { CountQuery.byJsonPath(TEST_TABLE) }
    }
}
