package solutions.bitbadger.documents.groovy.tests

import org.junit.jupiter.api.AfterEach
import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.Test
import solutions.bitbadger.documents.DocumentException
import solutions.bitbadger.documents.Field
import solutions.bitbadger.documents.query.DeleteQuery

import static Types.TEST_TABLE
import static org.junit.jupiter.api.Assertions.*

/**
 * Unit tests for the `Delete` object
 */
@DisplayName('Groovy | Query | DeleteQuery')
class DeleteQueryTest {

    /**
     * Clear the connection string (resets Dialect)
     */
    @AfterEach
    void cleanUp() {
        ForceDialect.none()
    }

    @Test
    @DisplayName('byId generates correctly | PostgreSQL')
    void byIdPostgres() {
        ForceDialect.postgres()
        assertEquals("DELETE FROM $TEST_TABLE WHERE data->>'id' = :id".toString(), DeleteQuery.byId(TEST_TABLE),
                'Delete query not constructed correctly')
    }

    @Test
    @DisplayName('byId generates correctly | SQLite')
    void byIdSQLite() {
        ForceDialect.sqlite()
        assertEquals("DELETE FROM $TEST_TABLE WHERE data->>'id' = :id".toString(), DeleteQuery.byId(TEST_TABLE),
                'Delete query not constructed correctly')
    }

    @Test
    @DisplayName('byFields generates correctly | PostgreSQL')
    void byFieldsPostgres() {
        ForceDialect.postgres()
        assertEquals("DELETE FROM $TEST_TABLE WHERE data->>'a' = :b".toString(),
                DeleteQuery.byFields(TEST_TABLE, List.of(Field.equal('a', '', ':b'))),
                'Delete query not constructed correctly')
    }

    @Test
    @DisplayName('byFields generates correctly | SQLite')
    void byFieldsSQLite() {
        ForceDialect.sqlite()
        assertEquals("DELETE FROM $TEST_TABLE WHERE data->>'a' = :b".toString(),
                DeleteQuery.byFields(TEST_TABLE, List.of(Field.equal('a', '', ':b'))),
                'Delete query not constructed correctly')
    }

    @Test
    @DisplayName('byContains generates correctly | PostgreSQL')
    void byContainsPostgres() {
        ForceDialect.postgres()
        assertEquals("DELETE FROM $TEST_TABLE WHERE data @> :criteria".toString(), DeleteQuery.byContains(TEST_TABLE),
                'Delete query not constructed correctly')
    }

    @Test
    @DisplayName('byContains fails | SQLite')
    void byContainsSQLite() {
        ForceDialect.sqlite()
        assertThrows(DocumentException) { DeleteQuery.byContains(TEST_TABLE) }
    }

    @Test
    @DisplayName('byJsonPath generates correctly | PostgreSQL')
    void byJsonPathPostgres() {
        ForceDialect.postgres()
        assertEquals("DELETE FROM $TEST_TABLE WHERE jsonb_path_exists(data, :path::jsonpath)".toString(),
                DeleteQuery.byJsonPath(TEST_TABLE), 'Delete query not constructed correctly')
    }

    @Test
    @DisplayName('byJsonPath fails | SQLite')
    void byJsonPathSQLite() {
        ForceDialect.sqlite()
        assertThrows(DocumentException) { DeleteQuery.byJsonPath(TEST_TABLE) }
    }
}
