package solutions.bitbadger.documents.groovy.tests

import org.junit.jupiter.api.AfterEach
import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.Test
import solutions.bitbadger.documents.DocumentException
import solutions.bitbadger.documents.Field
import solutions.bitbadger.documents.query.FindQuery

import static Types.TEST_TABLE
import static org.junit.jupiter.api.Assertions.*

/**
 * Unit tests for the `Find` object
 */
@DisplayName('Groovy | Query | FindQuery')
class FindQueryTest {

    /**
     * Clear the connection string (resets Dialect)
     */
    @AfterEach
    void cleanUp() {
        ForceDialect.none()
    }

    @Test
    @DisplayName('all generates correctly')
    void all() {
        assertEquals("SELECT data FROM $TEST_TABLE".toString(), FindQuery.all(TEST_TABLE),
                'Find query not constructed correctly')
    }

    @Test
    @DisplayName('byId generates correctly | PostgreSQL')
    void byIdPostgres() {
        ForceDialect.postgres()
        assertEquals("SELECT data FROM $TEST_TABLE WHERE data->>'id' = :id".toString(), FindQuery.byId(TEST_TABLE),
                'Find query not constructed correctly')
    }

    @Test
    @DisplayName('byId generates correctly | SQLite')
    void byIdSQLite() {
        ForceDialect.sqlite()
        assertEquals("SELECT data FROM $TEST_TABLE WHERE data->>'id' = :id".toString(), FindQuery.byId(TEST_TABLE),
                'Find query not constructed correctly')
    }

    @Test
    @DisplayName('byFields generates correctly | PostgreSQL')
    void byFieldsPostgres() {
        ForceDialect.postgres()
        assertEquals("SELECT data FROM $TEST_TABLE WHERE data->>'a' = :b AND (data->>'c')::numeric < :d".toString(),
                FindQuery.byFields(TEST_TABLE, List.of(Field.equal('a', '', ':b'), Field.less('c', 14, ':d'))),
                'Find query not constructed correctly')
    }

    @Test
    @DisplayName('byFields generates correctly | SQLite')
    void byFieldsSQLite() {
        ForceDialect.sqlite()
        assertEquals("SELECT data FROM $TEST_TABLE WHERE data->>'a' = :b AND data->>'c' < :d".toString(),
                FindQuery.byFields(TEST_TABLE, List.of(Field.equal('a', '', ':b'), Field.less('c', 14, ':d'))),
                'Find query not constructed correctly')
    }

    @Test
    @DisplayName('byContains generates correctly | PostgreSQL')
    void byContainsPostgres() {
        ForceDialect.postgres()
        assertEquals("SELECT data FROM $TEST_TABLE WHERE data @> :criteria".toString(),
                FindQuery.byContains(TEST_TABLE), 'Find query not constructed correctly')
    }

    @Test
    @DisplayName('byContains fails | SQLite')
    void byContainsSQLite() {
        ForceDialect.sqlite()
        assertThrows(DocumentException) { FindQuery.byContains(TEST_TABLE) }
    }

    @Test
    @DisplayName('byJsonPath generates correctly | PostgreSQL')
    void byJsonPathPostgres() {
        ForceDialect.postgres()
        assertEquals("SELECT data FROM $TEST_TABLE WHERE jsonb_path_exists(data, :path::jsonpath)".toString(),
                FindQuery.byJsonPath(TEST_TABLE), 'Find query not constructed correctly')
    }

    @Test
    @DisplayName('byJsonPath fails | SQLite')
    void byJsonPathSQLite() {
        ForceDialect.sqlite()
        assertThrows(DocumentException) { FindQuery.byJsonPath(TEST_TABLE) }
    }
}
