package solutions.bitbadger.documents.groovy.tests

import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.Test
import solutions.bitbadger.documents.DocumentException
import solutions.bitbadger.documents.Parameter
import solutions.bitbadger.documents.ParameterType

import static org.junit.jupiter.api.Assertions.*

/**
 * Unit tests for the `Parameter` class
 */
@DisplayName('Groovy | Parameter')
class ParameterTest {

    @Test
    @DisplayName('Construction with colon-prefixed name')
    void ctorWithColon() {
        def p = new Parameter(':test', ParameterType.STRING, 'ABC')
        assertEquals ':test', p.name, 'Parameter name was incorrect'
        assertEquals ParameterType.STRING, p.type, 'Parameter type was incorrect'
        assertEquals 'ABC', p.value, 'Parameter value was incorrect'
    }

    @Test
    @DisplayName('Construction with at-sign-prefixed name')
    void ctorWithAtSign() {
        def p = new Parameter('@yo', ParameterType.NUMBER, null)
        assertEquals '@yo', p.name, 'Parameter name was incorrect'
        assertEquals ParameterType.NUMBER, p.type, 'Parameter type was incorrect'
        assertNull p.value, 'Parameter value was incorrect'
    }

    @Test
    @DisplayName('Construction fails with incorrect prefix')
    void ctorFailsForPrefix() {
        assertThrows(DocumentException) { new Parameter('it', ParameterType.JSON, '') }
    }
}
