package solutions.bitbadger.documents.groovy.tests.integration

import solutions.bitbadger.documents.DocumentIndex

import static org.junit.jupiter.api.Assertions.*
import static solutions.bitbadger.documents.groovy.tests.Types.TEST_TABLE

final class DefinitionFunctions {

    static void ensureTable(ThrowawayDatabase db) {
        assertFalse db.dbObjectExists('ensured'), 'The "ensured" table should not exist'
        assertFalse db.dbObjectExists('idx_ensured_key'), 'The PK index for the "ensured" table should not exist'
        db.conn.ensureTable 'ensured'
        assertTrue db.dbObjectExists('ensured'), 'The "ensured" table should exist'
        assertTrue db.dbObjectExists('idx_ensured_key'), 'The PK index for the "ensured" table should now exist'
    }

    static void ensureFieldIndex(ThrowawayDatabase db) {
        assertFalse db.dbObjectExists("idx_${TEST_TABLE}_test"), 'The test index should not exist'
        db.conn.ensureFieldIndex TEST_TABLE, 'test', List.of('id', 'category')
        assertTrue db.dbObjectExists("idx_${TEST_TABLE}_test"), 'The test index should now exist'
    }

    static void ensureDocumentIndexFull(ThrowawayDatabase db) {
        assertFalse db.dbObjectExists('doc_table'), 'The "doc_table" table should not exist'
        db.conn.ensureTable 'doc_table'
        assertTrue db.dbObjectExists('doc_table'), 'The "doc_table" table should exist'
        assertFalse db.dbObjectExists('idx_doc_table_document'), 'The document index should not exist'
        db.conn.ensureDocumentIndex 'doc_table', DocumentIndex.FULL
        assertTrue db.dbObjectExists('idx_doc_table_document'), 'The document index should exist'
    }

    static void ensureDocumentIndexOptimized(ThrowawayDatabase db) {
        assertFalse db.dbObjectExists('doc_table'), 'The "doc_table" table should not exist'
        db.conn.ensureTable 'doc_table'
        assertTrue db.dbObjectExists('doc_table'), 'The "doc_table" table should exist'
        assertFalse db.dbObjectExists('idx_doc_table_document'), 'The document index should not exist'
        db.conn.ensureDocumentIndex 'doc_table', DocumentIndex.OPTIMIZED
        assertTrue db.dbObjectExists('idx_doc_table_document'), 'The document index should exist'
    }
}
