package solutions.bitbadger.documents.groovy.tests.integration

import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.Test

/**
 * PostgreSQL integration tests for the `Document` object / `insert`, `save`, `update` connection extension functions
 */
@DisplayName('Groovy | PostgreSQL: Document')
final class PostgreSQLDocumentIT {

    @Test
    @DisplayName('insert works with default values')
    void insertDefault() {
        new PgDB().withCloseable DocumentFunctions.&insertDefault
    }
    
    @Test
    @DisplayName('insert fails with duplicate key')
    void insertDupe() {
        new PgDB().withCloseable DocumentFunctions.&insertDupe
    }
    
    @Test
    @DisplayName('insert succeeds with numeric auto IDs')
    void insertNumAutoId() {
        new PgDB().withCloseable DocumentFunctions.&insertNumAutoId
    }
    
    @Test
    @DisplayName('insert succeeds with UUID auto ID')
    void insertUUIDAutoId() {
        new PgDB().withCloseable DocumentFunctions.&insertUUIDAutoId
    }
    
    @Test
    @DisplayName('insert succeeds with random string auto ID')
    void insertStringAutoId() {
        new PgDB().withCloseable DocumentFunctions.&insertStringAutoId
    }
    
    @Test
    @DisplayName('save updates an existing document')
    void saveMatch() {
        new PgDB().withCloseable DocumentFunctions.&saveMatch
    }
    
    @Test
    @DisplayName('save inserts a new document')
    void saveNoMatch() {
        new PgDB().withCloseable DocumentFunctions.&saveNoMatch
    }
    
    @Test
    @DisplayName('update replaces an existing document')
    void updateMatch() {
        new PgDB().withCloseable DocumentFunctions.&updateMatch
    }
    
    @Test
    @DisplayName('update succeeds when no document exists')
    void updateNoMatch() {
        new PgDB().withCloseable DocumentFunctions.&updateNoMatch
    }
}
