package solutions.bitbadger.documents.groovy.tests.integration

import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.Test

/**
 * PostgreSQL integration tests for the `Json` object / `json*` connection extension functions
 */
@DisplayName('Groovy | PostgreSQL: Json')
final class PostgreSQLJsonIT {

    @Test
    @DisplayName('all retrieves all documents')
    void allDefault() {
        new PgDB().withCloseable JsonFunctions.&allDefault
    }

    @Test
    @DisplayName('all succeeds with an empty table')
    void allEmpty() {
        new PgDB().withCloseable JsonFunctions.&allEmpty
    }

    @Test
    @DisplayName('byId retrieves a document via a string ID')
    void byIdString() {
        new PgDB().withCloseable JsonFunctions.&byIdString
    }

    @Test
    @DisplayName('byId retrieves a document via a numeric ID')
    void byIdNumber() {
        new PgDB().withCloseable JsonFunctions.&byIdNumber
    }

    @Test
    @DisplayName('byId returns null when a matching ID is not found')
    void byIdNotFound() {
        new PgDB().withCloseable JsonFunctions.&byIdNotFound
    }

    @Test
    @DisplayName('byFields retrieves matching documents')
    void byFieldsMatch() {
        new PgDB().withCloseable JsonFunctions.&byFieldsMatch
    }

    @Test
    @DisplayName('byFields retrieves ordered matching documents')
    void byFieldsMatchOrdered() {
        new PgDB().withCloseable JsonFunctions.&byFieldsMatchOrdered
    }

    @Test
    @DisplayName('byFields retrieves matching documents with a numeric IN clause')
    void byFieldsMatchNumIn() {
        new PgDB().withCloseable JsonFunctions.&byFieldsMatchNumIn
    }

    @Test
    @DisplayName('byFields succeeds when no documents match')
    void byFieldsNoMatch() {
        new PgDB().withCloseable JsonFunctions.&byFieldsNoMatch
    }

    @Test
    @DisplayName('byFields retrieves matching documents with an IN_ARRAY comparison')
    void byFieldsMatchInArray() {
        new PgDB().withCloseable JsonFunctions.&byFieldsMatchInArray
    }

    @Test
    @DisplayName('byFields succeeds when no documents match an IN_ARRAY comparison')
    void byFieldsNoMatchInArray() {
        new PgDB().withCloseable JsonFunctions.&byFieldsNoMatchInArray
    }

    @Test
    @DisplayName('byContains retrieves matching documents')
    void byContainsMatch() {
        new PgDB().withCloseable JsonFunctions.&byContainsMatch
    }

    @Test
    @DisplayName('byContains retrieves ordered matching documents')
    void byContainsMatchOrdered() {
        new PgDB().withCloseable JsonFunctions.&byContainsMatchOrdered
    }

    @Test
    @DisplayName('byContains succeeds when no documents match')
    void byContainsNoMatch() {
        new PgDB().withCloseable JsonFunctions.&byContainsNoMatch
    }

    @Test
    @DisplayName('byJsonPath retrieves matching documents')
    void byJsonPathMatch() {
        new PgDB().withCloseable JsonFunctions.&byJsonPathMatch
    }

    @Test
    @DisplayName('byJsonPath retrieves ordered matching documents')
    void byJsonPathMatchOrdered() {
        new PgDB().withCloseable JsonFunctions.&byJsonPathMatchOrdered
    }

    @Test
    @DisplayName('byJsonPath succeeds when no documents match')
    void byJsonPathNoMatch() {
        new PgDB().withCloseable JsonFunctions.&byJsonPathNoMatch
    }

    @Test
    @DisplayName('firstByFields retrieves a matching document')
    void firstByFieldsMatchOne() {
        new PgDB().withCloseable JsonFunctions.&firstByFieldsMatchOne
    }

    @Test
    @DisplayName('firstByFields retrieves a matching document among many')
    void firstByFieldsMatchMany() {
        new PgDB().withCloseable JsonFunctions.&firstByFieldsMatchMany
    }

    @Test
    @DisplayName('firstByFields retrieves a matching document among many (ordered)')
    void firstByFieldsMatchOrdered() {
        new PgDB().withCloseable JsonFunctions.&firstByFieldsMatchOrdered
    }

    @Test
    @DisplayName('firstByFields returns null when no document matches')
    void firstByFieldsNoMatch() {
        new PgDB().withCloseable JsonFunctions.&firstByFieldsNoMatch
    }

    @Test
    @DisplayName('firstByContains retrieves a matching document')
    void firstByContainsMatchOne() {
        new PgDB().withCloseable JsonFunctions.&firstByContainsMatchOne
    }

    @Test
    @DisplayName('firstByContains retrieves a matching document among many')
    void firstByContainsMatchMany() {
        new PgDB().withCloseable JsonFunctions.&firstByContainsMatchMany
    }

    @Test
    @DisplayName('firstByContains retrieves a matching document among many (ordered)')
    void firstByContainsMatchOrdered() {
        new PgDB().withCloseable JsonFunctions.&firstByContainsMatchOrdered
    }

    @Test
    @DisplayName('firstByContains returns null when no document matches')
    void firstByContainsNoMatch() {
        new PgDB().withCloseable JsonFunctions.&firstByContainsNoMatch
    }

    @Test
    @DisplayName('firstByJsonPath retrieves a matching document')
    void firstByJsonPathMatchOne() {
        new PgDB().withCloseable JsonFunctions.&firstByJsonPathMatchOne
    }

    @Test
    @DisplayName('firstByJsonPath retrieves a matching document among many')
    void firstByJsonPathMatchMany() {
        new PgDB().withCloseable JsonFunctions.&firstByJsonPathMatchMany
    }

    @Test
    @DisplayName('firstByJsonPath retrieves a matching document among many (ordered)')
    void firstByJsonPathMatchOrdered() {
        new PgDB().withCloseable JsonFunctions.&firstByJsonPathMatchOrdered
    }

    @Test
    @DisplayName('firstByJsonPath returns null when no document matches')
    void firstByJsonPathNoMatch() {
        new PgDB().withCloseable JsonFunctions.&firstByJsonPathNoMatch
    }

    @Test
    @DisplayName('writeAll retrieves all documents')
    void writeAllDefault() {
        new PgDB().withCloseable JsonFunctions.&writeAllDefault
    }

    @Test
    @DisplayName('writeAll succeeds with an empty table')
    void writeAllEmpty() {
        new PgDB().withCloseable JsonFunctions.&writeAllEmpty
    }

    @Test
    @DisplayName('writeById retrieves a document via a string ID')
    void writeByIdString() {
        new PgDB().withCloseable JsonFunctions.&writeByIdString
    }

    @Test
    @DisplayName('writeById retrieves a document via a numeric ID')
    void writeByIdNumber() {
        new PgDB().withCloseable JsonFunctions.&writeByIdNumber
    }

    @Test
    @DisplayName('writeById returns null when a matching ID is not found')
    void writeByIdNotFound() {
        new PgDB().withCloseable JsonFunctions.&writeByIdNotFound
    }

    @Test
    @DisplayName('writeByFields retrieves matching documents')
    void writeByFieldsMatch() {
        new PgDB().withCloseable JsonFunctions.&writeByFieldsMatch
    }

    @Test
    @DisplayName('writeByFields retrieves ordered matching documents')
    void writeByFieldsMatchOrdered() {
        new PgDB().withCloseable JsonFunctions.&writeByFieldsMatchOrdered
    }

    @Test
    @DisplayName('writeByFields retrieves matching documents with a numeric IN clause')
    void writeByFieldsMatchNumIn() {
        new PgDB().withCloseable JsonFunctions.&writeByFieldsMatchNumIn
    }

    @Test
    @DisplayName('writeByFields succeeds when no documents match')
    void writeByFieldsNoMatch() {
        new PgDB().withCloseable JsonFunctions.&writeByFieldsNoMatch
    }

    @Test
    @DisplayName('writeByFields retrieves matching documents with an IN_ARRAY comparison')
    void writeByFieldsMatchInArray() {
        new PgDB().withCloseable JsonFunctions.&writeByFieldsMatchInArray
    }

    @Test
    @DisplayName('writeByFields succeeds when no documents match an IN_ARRAY comparison')
    void writeByFieldsNoMatchInArray() {
        new PgDB().withCloseable JsonFunctions.&writeByFieldsNoMatchInArray
    }

    @Test
    @DisplayName('writeByContains retrieves matching documents')
    void writeByContainsMatch() {
        new PgDB().withCloseable JsonFunctions.&writeByContainsMatch
    }

    @Test
    @DisplayName('writeByContains retrieves ordered matching documents')
    void writeByContainsMatchOrdered() {
        new PgDB().withCloseable JsonFunctions.&writeByContainsMatchOrdered
    }

    @Test
    @DisplayName('writeByContains succeeds when no documents match')
    void writeByContainsNoMatch() {
        new PgDB().withCloseable JsonFunctions.&writeByContainsNoMatch
    }

    @Test
    @DisplayName('writeByJsonPath retrieves matching documents')
    void writeByJsonPathMatch() {
        new PgDB().withCloseable JsonFunctions.&writeByJsonPathMatch
    }

    @Test
    @DisplayName('writeByJsonPath retrieves ordered matching documents')
    void writeByJsonPathMatchOrdered() {
        new PgDB().withCloseable JsonFunctions.&writeByJsonPathMatchOrdered
    }

    @Test
    @DisplayName('writeByJsonPath succeeds when no documents match')
    void writeByJsonPathNoMatch() {
        new PgDB().withCloseable JsonFunctions.&writeByJsonPathNoMatch
    }

    @Test
    @DisplayName('writeFirstByFields retrieves a matching document')
    void writeFirstByFieldsMatchOne() {
        new PgDB().withCloseable JsonFunctions.&writeFirstByFieldsMatchOne
    }

    @Test
    @DisplayName('writeFirstByFields retrieves a matching document among many')
    void writeFirstByFieldsMatchMany() {
        new PgDB().withCloseable JsonFunctions.&writeFirstByFieldsMatchMany
    }

    @Test
    @DisplayName('writeFirstByFields retrieves a matching document among many (ordered)')
    void writeFirstByFieldsMatchOrdered() {
        new PgDB().withCloseable JsonFunctions.&writeFirstByFieldsMatchOrdered
    }

    @Test
    @DisplayName('writeFirstByFields returns null when no document matches')
    void writeFirstByFieldsNoMatch() {
        new PgDB().withCloseable JsonFunctions.&writeFirstByFieldsNoMatch
    }

    @Test
    @DisplayName('writeFirstByContains retrieves a matching document')
    void writeFirstByContainsMatchOne() {
        new PgDB().withCloseable JsonFunctions.&writeFirstByContainsMatchOne
    }

    @Test
    @DisplayName('writeFirstByContains retrieves a matching document among many')
    void writeFirstByContainsMatchMany() {
        new PgDB().withCloseable JsonFunctions.&writeFirstByContainsMatchMany
    }

    @Test
    @DisplayName('writeFirstByContains retrieves a matching document among many (ordered)')
    void writeFirstByContainsMatchOrdered() {
        new PgDB().withCloseable JsonFunctions.&writeFirstByContainsMatchOrdered
    }

    @Test
    @DisplayName('writeFirstByContains returns null when no document matches')
    void writeFirstByContainsNoMatch() {
        new PgDB().withCloseable JsonFunctions.&writeFirstByContainsNoMatch
    }

    @Test
    @DisplayName('writeFirstByJsonPath retrieves a matching document')
    void writeFirstByJsonPathMatchOne() {
        new PgDB().withCloseable JsonFunctions.&writeFirstByJsonPathMatchOne
    }

    @Test
    @DisplayName('writeFirstByJsonPath retrieves a matching document among many')
    void writeFirstByJsonPathMatchMany() {
        new PgDB().withCloseable JsonFunctions.&writeFirstByJsonPathMatchMany
    }

    @Test
    @DisplayName('writeFirstByJsonPath retrieves a matching document among many (ordered)')
    void writeFirstByJsonPathMatchOrdered() {
        new PgDB().withCloseable JsonFunctions.&writeFirstByJsonPathMatchOrdered
    }

    @Test
    @DisplayName('writeFirstByJsonPath returns null when no document matches')
    void writeFirstByJsonPathNoMatch() {
        new PgDB().withCloseable JsonFunctions.&writeFirstByJsonPathNoMatch
    }
}
