package solutions.bitbadger.documents.groovy.tests.integration

import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.Test

/**
 * PostgreSQL integration tests for the `RemoveFields` object / `removeFieldsBy*` connection extension functions
 */
@DisplayName('Groovy | PostgreSQL: RemoveFields')
final class PostgreSQLRemoveFieldsIT {

    @Test
    @DisplayName('byId removes fields from an existing document')
    void byIdMatchFields() {
        new PgDB().withCloseable RemoveFieldsFunctions.&byIdMatchFields
    }

    @Test
    @DisplayName('byId succeeds when fields do not exist on an existing document')
    void byIdMatchNoFields() {
        new PgDB().withCloseable RemoveFieldsFunctions.&byIdMatchNoFields
    }

    @Test
    @DisplayName('byId succeeds when no document exists')
    void byIdNoMatch() {
        new PgDB().withCloseable RemoveFieldsFunctions.&byIdNoMatch
    }

    @Test
    @DisplayName('byFields removes fields from matching documents')
    void byFieldsMatchFields() {
        new PgDB().withCloseable RemoveFieldsFunctions.&byFieldsMatchFields
    }

    @Test
    @DisplayName('byFields succeeds when fields do not exist on matching documents')
    void byFieldsMatchNoFields() {
        new PgDB().withCloseable RemoveFieldsFunctions.&byFieldsMatchNoFields
    }

    @Test
    @DisplayName('byFields succeeds when no matching documents exist')
    void byFieldsNoMatch() {
        new PgDB().withCloseable RemoveFieldsFunctions.&byFieldsNoMatch
    }

    @Test
    @DisplayName('byContains removes fields from matching documents')
    void byContainsMatchFields() {
        new PgDB().withCloseable RemoveFieldsFunctions.&byContainsMatchFields
    }

    @Test
    @DisplayName('byContains succeeds when fields do not exist on matching documents')
    void byContainsMatchNoFields() {
        new PgDB().withCloseable RemoveFieldsFunctions.&byContainsMatchNoFields
    }

    @Test
    @DisplayName('byContains succeeds when no matching documents exist')
    void byContainsNoMatch() {
        new PgDB().withCloseable RemoveFieldsFunctions.&byContainsNoMatch
    }

    @Test
    @DisplayName('byJsonPath removes fields from matching documents')
    void byJsonPathMatchFields() {
        new PgDB().withCloseable RemoveFieldsFunctions.&byJsonPathMatchFields
    }

    @Test
    @DisplayName('byJsonPath succeeds when fields do not exist on matching documents')
    void byJsonPathMatchNoFields() {
        new PgDB().withCloseable RemoveFieldsFunctions.&byJsonPathMatchNoFields
    }

    @Test
    @DisplayName('byJsonPath succeeds when no matching documents exist')
    void byJsonPathNoMatch() {
        new PgDB().withCloseable RemoveFieldsFunctions.&byJsonPathNoMatch
    }
}
