package solutions.bitbadger.documents.groovy.tests.integration

import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.Test
import solutions.bitbadger.documents.DocumentException

import static org.junit.jupiter.api.Assertions.assertThrows

/**
 * SQLite integration tests for the `Definition` object / `ensure*` connection extension functions
 */
@DisplayName('Groovy | SQLite: Definition')
final class SQLiteDefinitionIT {

    @Test
    @DisplayName('ensureTable creates table and index')
    void ensureTable() {
        new SQLiteDB().withCloseable DefinitionFunctions.&ensureTable
    }

    @Test
    @DisplayName('ensureFieldIndex creates an index')
    void ensureFieldIndex() {
        new SQLiteDB().withCloseable DefinitionFunctions.&ensureFieldIndex
    }

    @Test
    @DisplayName('ensureDocumentIndex fails for full index')
    void ensureDocumentIndexFull() {
        new SQLiteDB().withCloseable { db ->
            assertThrows(DocumentException) { DefinitionFunctions::ensureDocumentIndexFull db }
        }
    }

    @Test
    @DisplayName('ensureDocumentIndex fails for optimized index')
    void ensureDocumentIndexOptimized() {
        new SQLiteDB().withCloseable { db ->
            assertThrows(DocumentException) { DefinitionFunctions::ensureDocumentIndexOptimized db }
        }
    }
}
