package solutions.bitbadger.documents.groovy.tests.integration

import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.Test
import solutions.bitbadger.documents.DocumentException

import static org.junit.jupiter.api.Assertions.assertThrows

/**
 * SQLite integration tests for the `Patch` object / `patchBy*` connection extension functions
 */
@DisplayName('Groovy | SQLite: Patch')
final class SQLitePatchIT {

    @Test
    @DisplayName('byId patches an existing document')
    void byIdMatch() {
        new SQLiteDB().withCloseable PatchFunctions.&byIdMatch
    }

    @Test
    @DisplayName('byId succeeds for a non-existent document')
    void byIdNoMatch() {
        new SQLiteDB().withCloseable PatchFunctions.&byIdNoMatch
    }

    @Test
    @DisplayName('byFields patches matching document')
    void byFieldsMatch() {
        new SQLiteDB().withCloseable PatchFunctions.&byFieldsMatch
    }

    @Test
    @DisplayName('byFields succeeds when no documents match')
    void byFieldsNoMatch() {
        new SQLiteDB().withCloseable PatchFunctions.&byFieldsNoMatch
    }

    @Test
    @DisplayName('byContains fails')
    void byContainsFails() {
        new SQLiteDB().withCloseable { db ->
            assertThrows(DocumentException) { PatchFunctions.byContainsMatch db }
        }
    }

    @Test
    @DisplayName('byJsonPath fails')
    void byJsonPathFails() {
        new SQLiteDB().withCloseable { db ->
            assertThrows(DocumentException) { PatchFunctions.byJsonPathMatch db }
        }
    }
}
