package solutions.bitbadger.documents.kotlinx

import solutions.bitbadger.documents.DocumentException
import solutions.bitbadger.documents.DocumentIndex
import solutions.bitbadger.documents.java.Definition as JvmDefinition
import java.sql.Connection

/**
 * Functions to define tables and indexes
 */
object Definition {

    /**
     * Create a document table if necessary
     *
     * @param tableName The table whose existence should be ensured (may include schema)
     * @param conn The connection on which the query should be executed
     */
    fun ensureTable(tableName: String, conn: Connection) =
        JvmDefinition.ensureTable(tableName, conn)

    /**
     * Create a document table if necessary
     *
     * @param tableName The table whose existence should be ensured (may include schema)
     */
    fun ensureTable(tableName: String) =
        JvmDefinition.ensureTable(tableName)

    /**
     * Create an index on field(s) within documents in the specified table if necessary
     *
     * @param tableName The table to be indexed (may include schema)
     * @param indexName The name of the index to create
     * @param fields One or more fields to be indexed<
     * @param conn The connection on which the query should be executed
     */
    fun ensureFieldIndex(tableName: String, indexName: String, fields: Collection<String>, conn: Connection) =
        JvmDefinition.ensureFieldIndex(tableName, indexName, fields, conn)

    /**
     * Create an index on field(s) within documents in the specified table if necessary
     *
     * @param tableName The table to be indexed (may include schema)
     * @param indexName The name of the index to create
     * @param fields One or more fields to be indexed<
     */
    fun ensureFieldIndex(tableName: String, indexName: String, fields: Collection<String>) =
        JvmDefinition.ensureFieldIndex(tableName, indexName, fields)

    /**
     * Create a document index on a table (PostgreSQL only)
     *
     * @param tableName The table to be indexed (may include schema)
     * @param indexType The type of index to ensure
     * @param conn The connection on which the query should be executed
     * @throws DocumentException If called on a SQLite connection
     */
    fun ensureDocumentIndex(tableName: String, indexType: DocumentIndex, conn: Connection) =
        JvmDefinition.ensureDocumentIndex(tableName, indexType, conn)

    /**
     * Create a document index on a table (PostgreSQL only)
     *
     * @param tableName The table to be indexed (may include schema)
     * @param indexType The type of index to ensure
     * @throws DocumentException If called on a SQLite connection
     */
    fun ensureDocumentIndex(tableName: String, indexType: DocumentIndex) =
        JvmDefinition.ensureDocumentIndex(tableName, indexType)
}
