package solutions.bitbadger.documents.kotlinx.tests.integration

import solutions.bitbadger.documents.*
import solutions.bitbadger.documents.kotlinx.extensions.*
import solutions.bitbadger.documents.kotlinx.tests.JsonDocument
import solutions.bitbadger.documents.kotlinx.tests.TEST_TABLE
import kotlin.test.assertFalse
import kotlin.test.assertTrue

/**
 * Integration tests for the `Exists` object
 */
object ExistsFunctions {

    fun byIdMatch(db: ThrowawayDatabase) {
        JsonDocument.load(db)
        assertTrue("The document with ID \"three\" should exist") { db.conn.existsById(TEST_TABLE, "three") }
    }

    fun byIdNoMatch(db: ThrowawayDatabase) {
        JsonDocument.load(db)
        assertFalse("The document with ID \"seven\" should not exist") { db.conn.existsById(TEST_TABLE, "seven") }
    }

    fun byFieldsMatch(db: ThrowawayDatabase) {
        JsonDocument.load(db)
        assertTrue("Matching documents should have been found") {
            db.conn.existsByFields(TEST_TABLE, listOf(Field.equal("numValue", 10)))
        }
    }

    fun byFieldsNoMatch(db: ThrowawayDatabase) {
        JsonDocument.load(db)
        assertFalse("No matching documents should have been found") {
            db.conn.existsByFields(TEST_TABLE, listOf(Field.equal("nothing", "none")))
        }
    }

    fun byContainsMatch(db: ThrowawayDatabase) {
        JsonDocument.load(db)
        assertTrue("Matching documents should have been found") {
            db.conn.existsByContains(TEST_TABLE, mapOf("value" to "purple"))
        }
    }

    fun byContainsNoMatch(db: ThrowawayDatabase) {
        JsonDocument.load(db)
        assertFalse("Matching documents should not have been found") {
            db.conn.existsByContains(TEST_TABLE, mapOf("value" to "violet"))
        }
    }

    fun byJsonPathMatch(db: ThrowawayDatabase) {
        JsonDocument.load(db)
        assertTrue("Matching documents should have been found") {
            db.conn.existsByJsonPath(TEST_TABLE, "$.numValue ? (@ == 10)")
        }
    }

    fun byJsonPathNoMatch(db: ThrowawayDatabase) {
        JsonDocument.load(db)
        assertFalse("Matching documents should not have been found") {
            db.conn.existsByJsonPath(TEST_TABLE, "$.numValue ? (@ == 10.1)")
        }
    }
}
