package solutions.bitbadger.documents.kotlinx.tests.integration

import solutions.bitbadger.documents.*
import solutions.bitbadger.documents.kotlinx.Results
import solutions.bitbadger.documents.kotlinx.extensions.*
import solutions.bitbadger.documents.kotlinx.tests.TEST_TABLE

/**
 * A wrapper for a throwaway PostgreSQL database
 */
class PgDB : ThrowawayDatabase() {

    init {
        Configuration.connectionString = connString("postgres")
        Configuration.dbConn().use { it.customNonQuery("CREATE DATABASE $dbName") }
        Configuration.connectionString = connString(dbName)
    }

    override val conn = Configuration.dbConn()

    init {
        conn.ensureTable(TEST_TABLE)
    }

    override fun close() {
        conn.close()
        Configuration.connectionString = connString("postgres")
        Configuration.dbConn().use { it.customNonQuery("DROP DATABASE $dbName") }
        Configuration.connectionString = null
    }

    override fun dbObjectExists(name: String) =
        conn.customScalar("SELECT EXISTS (SELECT 1 FROM pg_class WHERE relname = :name) AS it",
            listOf(Parameter(":name", ParameterType.STRING, name)), Results::toExists)

    companion object {

        /**
         * Create a connection string for the given database
         *
         * @param database The database to which the library should connect
         * @return The connection string for the database
         */
        private fun connString(database: String) =
            "jdbc:postgresql://localhost/$database?user=postgres&password=postgres"
    }
}
