package solutions.bitbadger.documents.kotlinx.tests.integration

import org.junit.jupiter.api.DisplayName
import kotlin.test.Test

/**
 * PostgreSQL integration tests for the `Find` object / `find*` connection extension functions
 */
@DisplayName("KotlinX | PostgreSQL: Find")
class PostgreSQLFindIT {

    @Test
    @DisplayName("all retrieves all documents")
    fun allDefault() =
        PgDB().use(FindFunctions::allDefault)

    @Test
    @DisplayName("all sorts data ascending")
    fun allAscending() =
        PgDB().use(FindFunctions::allAscending)

    @Test
    @DisplayName("all sorts data descending")
    fun allDescending() =
        PgDB().use(FindFunctions::allDescending)

    @Test
    @DisplayName("all sorts data numerically")
    fun allNumOrder() =
        PgDB().use(FindFunctions::allNumOrder)

    @Test
    @DisplayName("all succeeds with an empty table")
    fun allEmpty() =
        PgDB().use(FindFunctions::allEmpty)

    @Test
    @DisplayName("byId retrieves a document via a string ID")
    fun byIdString() =
        PgDB().use(FindFunctions::byIdString)

    @Test
    @DisplayName("byId retrieves a document via a numeric ID")
    fun byIdNumber() =
        PgDB().use(FindFunctions::byIdNumber)

    @Test
    @DisplayName("byId returns null when a matching ID is not found")
    fun byIdNotFound() =
        PgDB().use(FindFunctions::byIdNotFound)

    @Test
    @DisplayName("byFields retrieves matching documents")
    fun byFieldsMatch() =
        PgDB().use(FindFunctions::byFieldsMatch)

    @Test
    @DisplayName("byFields retrieves ordered matching documents")
    fun byFieldsMatchOrdered() =
        PgDB().use(FindFunctions::byFieldsMatchOrdered)

    @Test
    @DisplayName("byFields retrieves matching documents with a numeric IN clause")
    fun byFieldsMatchNumIn() =
        PgDB().use(FindFunctions::byFieldsMatchNumIn)

    @Test
    @DisplayName("byFields succeeds when no documents match")
    fun byFieldsNoMatch() =
        PgDB().use(FindFunctions::byFieldsNoMatch)

    @Test
    @DisplayName("byFields retrieves matching documents with an IN_ARRAY comparison")
    fun byFieldsMatchInArray() =
        PgDB().use(FindFunctions::byFieldsMatchInArray)

    @Test
    @DisplayName("byFields succeeds when no documents match an IN_ARRAY comparison")
    fun byFieldsNoMatchInArray() =
        PgDB().use(FindFunctions::byFieldsNoMatchInArray)

    @Test
    @DisplayName("byContains retrieves matching documents")
    fun byContainsMatch() =
        PgDB().use(FindFunctions::byContainsMatch)

    @Test
    @DisplayName("byContains retrieves ordered matching documents")
    fun byContainsMatchOrdered() =
        PgDB().use(FindFunctions::byContainsMatchOrdered)

    @Test
    @DisplayName("byContains succeeds when no documents match")
    fun byContainsNoMatch() =
        PgDB().use(FindFunctions::byContainsNoMatch)

    @Test
    @DisplayName("byJsonPath retrieves matching documents")
    fun byJsonPathMatch() =
        PgDB().use(FindFunctions::byJsonPathMatch)

    @Test
    @DisplayName("byJsonPath retrieves ordered matching documents")
    fun byJsonPathMatchOrdered() =
        PgDB().use(FindFunctions::byJsonPathMatchOrdered)

    @Test
    @DisplayName("byJsonPath succeeds when no documents match")
    fun byJsonPathNoMatch() =
        PgDB().use(FindFunctions::byJsonPathNoMatch)

    @Test
    @DisplayName("firstByFields retrieves a matching document")
    fun firstByFieldsMatchOne() =
        PgDB().use(FindFunctions::firstByFieldsMatchOne)

    @Test
    @DisplayName("firstByFields retrieves a matching document among many")
    fun firstByFieldsMatchMany() =
        PgDB().use(FindFunctions::firstByFieldsMatchMany)

    @Test
    @DisplayName("firstByFields retrieves a matching document among many (ordered)")
    fun firstByFieldsMatchOrdered() =
        PgDB().use(FindFunctions::firstByFieldsMatchOrdered)

    @Test
    @DisplayName("firstByFields returns null when no document matches")
    fun firstByFieldsNoMatch() =
        PgDB().use(FindFunctions::firstByFieldsNoMatch)

    @Test
    @DisplayName("firstByContains retrieves a matching document")
    fun firstByContainsMatchOne() =
        PgDB().use(FindFunctions::firstByContainsMatchOne)

    @Test
    @DisplayName("firstByContains retrieves a matching document among many")
    fun firstByContainsMatchMany() =
        PgDB().use(FindFunctions::firstByContainsMatchMany)

    @Test
    @DisplayName("firstByContains retrieves a matching document among many (ordered)")
    fun firstByContainsMatchOrdered() =
        PgDB().use(FindFunctions::firstByContainsMatchOrdered)

    @Test
    @DisplayName("firstByContains returns null when no document matches")
    fun firstByContainsNoMatch() =
        PgDB().use(FindFunctions::firstByContainsNoMatch)

    @Test
    @DisplayName("firstByJsonPath retrieves a matching document")
    fun firstByJsonPathMatchOne() =
        PgDB().use(FindFunctions::firstByJsonPathMatchOne)

    @Test
    @DisplayName("firstByJsonPath retrieves a matching document among many")
    fun firstByJsonPathMatchMany() =
        PgDB().use(FindFunctions::firstByJsonPathMatchMany)

    @Test
    @DisplayName("firstByJsonPath retrieves a matching document among many (ordered)")
    fun firstByJsonPathMatchOrdered() =
        PgDB().use(FindFunctions::firstByJsonPathMatchOrdered)

    @Test
    @DisplayName("firstByJsonPath returns null when no document matches")
    fun firstByJsonPathNoMatch() =
        PgDB().use(FindFunctions::firstByJsonPathNoMatch)
}
