package solutions.bitbadger.documents.scala

import solutions.bitbadger.documents.DocumentIndex
import solutions.bitbadger.documents.java.Definition as CoreDefinition

import java.sql.Connection
import _root_.scala.jdk.CollectionConverters.*

object Definition:

  /**
   * Create a document table if necessary
   *
   * @param tableName The table whose existence should be ensured (may include schema)
   * @param conn      The connection on which the query should be executed
   * @throws DocumentException If the dialect is not configured
   */
  def ensureTable(tableName: String, conn: Connection): Unit =
    CoreDefinition.ensureTable(tableName, conn)

  /**
   * Create a document table if necessary
   *
   * @param tableName The table whose existence should be ensured (may include schema)
   * @throws DocumentException If no connection string has been set
   */
  def ensureTable(tableName: String): Unit =
    CoreDefinition.ensureTable(tableName)

  /**
   * Create an index on field(s) within documents in the specified table if necessary
   *
   * @param tableName The table to be indexed (may include schema)
   * @param indexName The name of the index to create
   * @param fields    One or more fields to be indexed
   * @param conn      The connection on which the query should be executed
   * @throws DocumentException If any dependent process does
   */
  def ensureFieldIndex(tableName: String, indexName: String, fields: Seq[String], conn: Connection): Unit =
    CoreDefinition.ensureFieldIndex(tableName, indexName, fields.asJava, conn)

  /**
   * Create an index on field(s) within documents in the specified table if necessary
   *
   * @param tableName The table to be indexed (may include schema)
   * @param indexName The name of the index to create
   * @param fields    One or more fields to be indexed
   * @throws DocumentException If no connection string has been set, or if any dependent process does
   */
  def ensureFieldIndex(tableName: String, indexName: String, fields: Seq[String]): Unit =
    CoreDefinition.ensureFieldIndex(tableName, indexName, fields.asJava)

  /**
   * Create a document index on a table (PostgreSQL only)
   *
   * @param tableName The table to be indexed (may include schema)
   * @param indexType The type of index to ensure
   * @param conn      The connection on which the query should be executed
   * @throws DocumentException If called on a SQLite connection
   */
  def ensureDocumentIndex(tableName: String, indexType: DocumentIndex, conn: Connection): Unit =
    CoreDefinition.ensureDocumentIndex(tableName, indexType, conn)

  /**
   * Create a document index on a table (PostgreSQL only)
   *
   * @param tableName The table to be indexed (may include schema)
   * @param indexType The type of index to ensure
   * @throws DocumentException If no connection string has been set, or if called on a SQLite connection
   */
  def ensureDocumentIndex(tableName: String, indexType: DocumentIndex): Unit =
    CoreDefinition.ensureDocumentIndex(tableName, indexType)
