package solutions.bitbadger.documents.scala

import solutions.bitbadger.documents.java.Document as CoreDocument

import java.sql.Connection

object Document:

  /**
   * Insert a new document
   *
   * @param tableName The table into which the document should be inserted (may include schema)
   * @param document  The document to be inserted
   * @param conn      The connection on which the query should be executed
   * @throws DocumentException If IDs are misconfigured, or if the database command fails
   */
  def insert[Doc](tableName: String, document: Doc, conn: Connection): Unit =
    CoreDocument.insert(tableName, document, conn)

  /**
   * Insert a new document (creates connection)
   *
   * @param tableName The table into which the document should be inserted (may include schema)
   * @param document  The document to be inserted
   * @throws DocumentException If IDs are misconfigured, or if the database command fails
   */
  def insert[Doc](tableName: String, document: Doc): Unit =
    CoreDocument.insert(tableName, document)

  /**
   * Save a document, inserting it if it does not exist and updating it if it does (AKA "upsert")
   *
   * @param tableName The table in which the document should be saved (may include schema)
   * @param document  The document to be saved
   * @param conn      The connection on which the query should be executed
   * @throws DocumentException If the database command fails
   */
  def save[Doc](tableName: String, document: Doc, conn: Connection): Unit =
    CoreDocument.save(tableName, document, conn)

  /**
   * Save a document, inserting it if it does not exist and updating it if it does (AKA "upsert"; creates connection)
   *
   * @param tableName The table in which the document should be saved (may include schema)
   * @param document  The document to be saved
   * @throws DocumentException If the database command fails
   */
  def save[Doc](tableName: String, document: Doc): Unit =
    CoreDocument.save(tableName, document)

  /**
   * Update (replace) a document by its ID
   *
   * @param tableName The table in which the document should be replaced (may include schema)
   * @param docId     The ID of the document to be replaced
   * @param document  The document to be replaced
   * @param conn      The connection on which the query should be executed
   * @throws DocumentException If no dialect has been configured, or if the database command fails
   */
  def update[Key, Doc](tableName: String, docId: Key, document: Doc, conn: Connection): Unit =
    CoreDocument.update(tableName, docId, document, conn)

  /**
   * Update (replace) a document by its ID (creates connection)
   *
   * @param tableName The table in which the document should be replaced (may include schema)
   * @param docId     The ID of the document to be replaced
   * @param document  The document to be replaced
   * @throws DocumentException If no dialect has been configured, or if the database command fails
   */
  def update[Key, Doc](tableName: String, docId: Key, document: Doc): Unit =
    CoreDocument.update(tableName, docId, document)
