package solutions.bitbadger.documents.scala.tests

import org.junit.jupiter.api.Assertions.*
import org.junit.jupiter.api.{DisplayName, Test}
import solutions.bitbadger.documents.{AutoId, Configuration, Dialect, DocumentException}

@DisplayName("Scala | Configuration")
class ConfigurationTest:

  @Test
  @DisplayName("Default ID field is `id`")
  def defaultIdField(): Unit =
    assertEquals("id", Configuration.idField, "Default ID field incorrect")

  @Test
  @DisplayName("Default Auto ID strategy is `DISABLED`")
  def defaultAutoId(): Unit =
    assertEquals(AutoId.DISABLED, Configuration.autoIdStrategy, "Default Auto ID strategy should be `disabled`")

  @Test
  @DisplayName("Default ID string length should be 16")
  def defaultIdStringLength(): Unit =
    assertEquals(16, Configuration.idStringLength, "Default ID string length should be 16")

  @Test
  @DisplayName("Dialect is derived from connection string")
  def dialectIsDerived(): Unit =
    try
      assertThrows(classOf[DocumentException], () => Configuration.dialect())
      Configuration.setConnectionString("jdbc:postgresql:db")
      assertEquals(Dialect.POSTGRESQL, Configuration.dialect())
    finally
      Configuration.setConnectionString(null)
