package solutions.bitbadger.documents.scala.tests.integration

import org.junit.jupiter.api.Assertions.*
import solutions.bitbadger.documents.DocumentIndex
import solutions.bitbadger.documents.scala.extensions.*
import solutions.bitbadger.documents.scala.tests.TEST_TABLE

object DefinitionFunctions:

  def ensureTable(db: ThrowawayDatabase): Unit =
    assertFalse(db.dbObjectExists("ensured"), "The 'ensured' table should not exist")
    assertFalse(db.dbObjectExists("idx_ensured_key"), "The PK index for the 'ensured' table should not exist")
    db.conn.ensureTable("ensured")
    assertTrue(db.dbObjectExists("ensured"), "The 'ensured' table should exist")
    assertTrue(db.dbObjectExists("idx_ensured_key"), "The PK index for the 'ensured' table should now exist")

  def ensureFieldIndex(db: ThrowawayDatabase): Unit = 
    assertFalse(db.dbObjectExists("idx_${TEST_TABLE}_test"), "The test index should not exist")
    db.conn.ensureFieldIndex(TEST_TABLE, "test", "id" :: "category" :: Nil)
    assertTrue(db.dbObjectExists(s"idx_${TEST_TABLE}_test"), "The test index should now exist")

  def ensureDocumentIndexFull(db: ThrowawayDatabase): Unit =
    assertFalse(db.dbObjectExists("doc_table"), "The 'doc_table' table should not exist")
    db.conn.ensureTable("doc_table")
    assertTrue(db.dbObjectExists("doc_table"), "The 'doc_table' table should exist")
    assertFalse(db.dbObjectExists("idx_doc_table_document"), "The document index should not exist")
    db.conn.ensureDocumentIndex("doc_table", DocumentIndex.FULL)
    assertTrue(db.dbObjectExists("idx_doc_table_document"), "The document index should exist")

  def ensureDocumentIndexOptimized(db: ThrowawayDatabase): Unit =
    assertFalse(db.dbObjectExists("doc_table"), "The 'doc_table' table should not exist")
    db.conn.ensureTable("doc_table")
    assertTrue(db.dbObjectExists("doc_table"), "The 'doc_table' table should exist")
    assertFalse(db.dbObjectExists("idx_doc_table_document"), "The document index should not exist")
    db.conn.ensureDocumentIndex("doc_table", DocumentIndex.OPTIMIZED)
    assertTrue(db.dbObjectExists("idx_doc_table_document"), "The document index should exist")
