package solutions.bitbadger.documents.scala.tests.integration

import org.junit.jupiter.api.Assertions.*
import solutions.bitbadger.documents.Field
import solutions.bitbadger.documents.scala.extensions.*
import solutions.bitbadger.documents.scala.tests.TEST_TABLE

object DeleteFunctions:

  def byIdMatch(db: ThrowawayDatabase): Unit =
    JsonDocument.load(db)
    assertEquals(5L, db.conn.countAll(TEST_TABLE), "There should be 5 documents in the table")
    db.conn.deleteById(TEST_TABLE, "four")
    assertEquals(4L, db.conn.countAll(TEST_TABLE), "There should now be 4 documents in the table")

  def byIdNoMatch(db: ThrowawayDatabase): Unit =
    JsonDocument.load(db)
    assertEquals(5L, db.conn.countAll(TEST_TABLE), "There should be 5 documents in the table")
    db.conn.deleteById(TEST_TABLE, "negative four")
    assertEquals(5L, db.conn.countAll(TEST_TABLE), "There should still be 5 documents in the table")

  def byFieldsMatch(db: ThrowawayDatabase): Unit =
    JsonDocument.load(db)
    assertEquals(5L, db.conn.countAll(TEST_TABLE), "There should be 5 documents in the table")
    db.conn.deleteByFields(TEST_TABLE, Field.notEqual("value", "purple") :: Nil)
    assertEquals(2L, db.conn.countAll(TEST_TABLE), "There should now be 2 documents in the table")

  def byFieldsNoMatch(db: ThrowawayDatabase): Unit =
    JsonDocument.load(db)
    assertEquals(5L, db.conn.countAll(TEST_TABLE), "There should be 5 documents in the table")
    db.conn.deleteByFields(TEST_TABLE, Field.equal("value", "crimson") :: Nil)
    assertEquals(5L, db.conn.countAll(TEST_TABLE), "There should still be 5 documents in the table")

  def byContainsMatch(db: ThrowawayDatabase): Unit =
    JsonDocument.load(db)
    assertEquals(5L, db.conn.countAll(TEST_TABLE), "There should be 5 documents in the table")
    db.conn.deleteByContains(TEST_TABLE, Map.Map1("value", "purple"))
    assertEquals(3L, db.conn.countAll(TEST_TABLE), "There should now be 3 documents in the table")

  def byContainsNoMatch(db: ThrowawayDatabase): Unit =
    JsonDocument.load(db)
    assertEquals(5L, db.conn.countAll(TEST_TABLE), "There should be 5 documents in the table")
    db.conn.deleteByContains(TEST_TABLE, Map.Map1("target", "acquired"))
    assertEquals(5L, db.conn.countAll(TEST_TABLE), "There should still be 5 documents in the table")

  def byJsonPathMatch(db: ThrowawayDatabase): Unit =
    JsonDocument.load(db)
    assertEquals(5L, db.conn.countAll(TEST_TABLE), "There should be 5 documents in the table")
    db.conn.deleteByJsonPath(TEST_TABLE, "$.value ? (@ == \"purple\")")
    assertEquals(3L, db.conn.countAll(TEST_TABLE), "There should now be 3 documents in the table")

  def byJsonPathNoMatch(db: ThrowawayDatabase): Unit =
    JsonDocument.load(db)
    assertEquals(5L, db.conn.countAll(TEST_TABLE), "There should be 5 documents in the table")
    db.conn.deleteByJsonPath(TEST_TABLE, "$.numValue ? (@ > 100)")
    assertEquals(5L, db.conn.countAll(TEST_TABLE), "There should still be 5 documents in the table")
