package solutions.bitbadger.documents.scala.tests.integration

import org.junit.jupiter.api.Assertions.*
import solutions.bitbadger.documents.Field
import solutions.bitbadger.documents.scala.extensions.*
import solutions.bitbadger.documents.scala.tests.TEST_TABLE

object ExistsFunctions:
  
  def byIdMatch(db: ThrowawayDatabase): Unit =
    JsonDocument.load(db)
    assertTrue(db.conn.existsById(TEST_TABLE, "three"), "The document with ID \"three\" should exist")

  def byIdNoMatch(db: ThrowawayDatabase): Unit =
    JsonDocument.load(db)
    assertFalse(db.conn.existsById(TEST_TABLE, "seven"), "The document with ID \"seven\" should not exist")

  def byFieldsMatch(db: ThrowawayDatabase): Unit =
    JsonDocument.load(db)
    assertTrue(db.conn.existsByFields(TEST_TABLE, Field.equal("numValue", 10) :: Nil),
      "Matching documents should have been found")

  def byFieldsNoMatch(db: ThrowawayDatabase): Unit =
    JsonDocument.load(db)
    assertFalse(db.conn.existsByFields(TEST_TABLE, Field.equal("nothing", "none") :: Nil),
      "No matching documents should have been found")

  def byContainsMatch(db: ThrowawayDatabase): Unit =
    JsonDocument.load(db)
    assertTrue(db.conn.existsByContains(TEST_TABLE, Map.Map1("value", "purple")),
      "Matching documents should have been found")

  def byContainsNoMatch(db: ThrowawayDatabase): Unit =
    JsonDocument.load(db)
    assertFalse(db.conn.existsByContains(TEST_TABLE, Map.Map1("value", "violet")),
      "Matching documents should not have been found")

  def byJsonPathMatch(db: ThrowawayDatabase): Unit =
    JsonDocument.load(db)
    assertTrue(db.conn.existsByJsonPath(TEST_TABLE, "$.numValue ? (@ == 10)"),
      "Matching documents should have been found")

  def byJsonPathNoMatch(db: ThrowawayDatabase): Unit =
    JsonDocument.load(db)
    assertFalse(db.conn.existsByJsonPath(TEST_TABLE, "$.numValue ? (@ == 10.1)"),
      "Matching documents should not have been found")
