package solutions.bitbadger.documents.scala.tests.integration

import solutions.bitbadger.documents.scala.extensions.insert
import solutions.bitbadger.documents.scala.tests.TEST_TABLE

class JsonDocument(val id: String = "", val value: String = "", val numValue: Int = 0, val sub: SubDocument = null)

object JsonDocument:

  /** Documents to use for testing */
  private val testDocuments = List(
    JsonDocument("one", "FIRST!", 0, null),
    JsonDocument("two", "another", 10, SubDocument("green", "blue")),
    JsonDocument("three", "", 4, null),
    JsonDocument("four", "purple", 17, SubDocument("green", "red")),
    JsonDocument("five", "purple", 18, null))

  def load(db: ThrowawayDatabase, tableName: String = TEST_TABLE): Unit =
    testDocuments.foreach { it => db.conn.insert(tableName, it) }

  /** Document ID `one` as a JSON string */
  val one = """{"id":"one","value":"FIRST!","numValue":0,"sub":null}"""

  /** Document ID `two` as a JSON string */
  val two = """{"id":"two","value":"another","numValue":10,"sub":{"foo":"green","bar":"blue"}}"""

  /** Document ID `three` as a JSON string */
  val three = """{"id":"three","value":"","numValue":4,"sub":null}"""

  /** Document ID `four` as a JSON string */
  val four = """{"id":"four","value":"purple","numValue":17,"sub":{"foo":"green","bar":"red"}}"""

  /** Document ID `five` as a JSON string */
  val five = """{"id":"five","value":"purple","numValue":18,"sub":null}"""
