package solutions.bitbadger.documents.scala.tests.integration

import org.junit.jupiter.api.Assertions.*
import solutions.bitbadger.documents.Field
import solutions.bitbadger.documents.scala.extensions.*
import solutions.bitbadger.documents.scala.tests.TEST_TABLE

object PatchFunctions:

  def byIdMatch(db: ThrowawayDatabase): Unit =
    JsonDocument.load(db)
    db.conn.patchById(TEST_TABLE, "one", Map.Map1("numValue", 44))
    val doc = db.conn.findById[String, JsonDocument](TEST_TABLE, "one")
    assertTrue(doc.isDefined, "There should have been a document returned")
    assertEquals("one", doc.get.id, "An incorrect document was returned")
    assertEquals(44, doc.get.numValue, "The document was not patched")

  def byIdNoMatch(db: ThrowawayDatabase): Unit =
    JsonDocument.load(db)
    assertFalse(db.conn.existsById(TEST_TABLE, "forty-seven"), "Document with ID \"forty-seven\" should not exist")
    db.conn.patchById(TEST_TABLE, "forty-seven", Map.Map1("foo", "green")) // no exception = pass

  def byFieldsMatch(db: ThrowawayDatabase): Unit =
    JsonDocument.load(db)
    db.conn.patchByFields(TEST_TABLE, Field.equal("value", "purple") :: Nil, Map.Map1("numValue", 77))
    assertEquals(2L, db.conn.countByFields(TEST_TABLE, Field.equal("numValue", 77) :: Nil),
      "There should have been 2 documents with numeric value 77")

  def byFieldsNoMatch(db: ThrowawayDatabase): Unit =
    JsonDocument.load(db)
    val fields = Field.equal("value", "burgundy") :: Nil
    assertFalse(db.conn.existsByFields(TEST_TABLE, fields), "There should be no documents with value of \"burgundy\"")
    db.conn.patchByFields(TEST_TABLE, fields, Map.Map1("foo", "green")) // no exception = pass

  def byContainsMatch(db: ThrowawayDatabase): Unit =
    JsonDocument.load(db)
    val contains = Map.Map1("value", "another")
    db.conn.patchByContains(TEST_TABLE, contains, Map.Map1("numValue", 12))
    val doc = db.conn.findFirstByContains[JsonDocument, Map.Map1[String, String]](TEST_TABLE, contains)
    assertTrue(doc.isDefined, "There should have been a document returned")
    assertEquals("two", doc.get.id, "The incorrect document was returned")
    assertEquals(12, doc.get.numValue, "The document was not updated")

  def byContainsNoMatch(db: ThrowawayDatabase): Unit =
    JsonDocument.load(db)
    val contains = Map.Map1("value", "updated")
    assertFalse(db.conn.existsByContains(TEST_TABLE, contains), "There should be no matching documents")
    db.conn.patchByContains(TEST_TABLE, contains, Map.Map1("sub.foo", "green")) // no exception = pass

  def byJsonPathMatch(db: ThrowawayDatabase): Unit =
    JsonDocument.load(db)
    val path = "$.numValue ? (@ > 10)"
    db.conn.patchByJsonPath(TEST_TABLE, path, Map.Map1("value", "blue"))
    val docs = db.conn.findByJsonPath[JsonDocument](TEST_TABLE, path)
    assertEquals(2, docs.size, "There should have been two documents returned")
    docs.foreach { doc =>
      assertTrue(("four" :: "five" :: Nil).contains(doc.id), s"An incorrect document was returned (${doc.id})")
      assertEquals("blue", doc.value, s"The value for ID ${doc.id} was incorrect")
    }

  def byJsonPathNoMatch(db: ThrowawayDatabase): Unit =
    JsonDocument.load(db)
    val path = "$.numValue ? (@ > 100)"
    assertFalse(db.conn.existsByJsonPath(TEST_TABLE, path), "There should be no documents with numeric values over 100")
    db.conn.patchByJsonPath(TEST_TABLE, path, Map.Map1("value", "blue")) // no exception = pass
