package solutions.bitbadger.documents.scala.tests.integration

import org.junit.jupiter.api.{DisplayName, Test}

import scala.util.Using

/**
 * PostgreSQL integration tests for the `RemoveFields` object / `removeFieldsBy*` connection extension functions
 */
@DisplayName("Scala | PostgreSQL: RemoveFields")
class PostgreSQLRemoveFieldsIT:

  @Test
  @DisplayName("byId removes fields from an existing document")
  def byIdMatchFields(): Unit =
    Using(PgDB()) { db => RemoveFieldsFunctions. byIdMatchFields(db) }

  @Test
  @DisplayName("byId succeeds when fields do not exist on an existing document")
  def byIdMatchNoFields(): Unit =
    Using(PgDB()) { db => RemoveFieldsFunctions. byIdMatchNoFields(db) }

  @Test
  @DisplayName("byId succeeds when no document exists")
  def byIdNoMatch(): Unit =
    Using(PgDB()) { db => RemoveFieldsFunctions. byIdNoMatch(db) }

  @Test
  @DisplayName("byFields removes fields from matching documents")
  def byFieldsMatchFields(): Unit =
    Using(PgDB()) { db => RemoveFieldsFunctions. byFieldsMatchFields(db) }

  @Test
  @DisplayName("byFields succeeds when fields do not exist on matching documents")
  def byFieldsMatchNoFields(): Unit =
    Using(PgDB()) { db => RemoveFieldsFunctions. byFieldsMatchNoFields(db) }

  @Test
  @DisplayName("byFields succeeds when no matching documents exist")
  def byFieldsNoMatch(): Unit =
    Using(PgDB()) { db => RemoveFieldsFunctions. byFieldsNoMatch(db) }

  @Test
  @DisplayName("byContains removes fields from matching documents")
  def byContainsMatchFields(): Unit =
    Using(PgDB()) { db => RemoveFieldsFunctions. byContainsMatchFields(db) }

  @Test
  @DisplayName("byContains succeeds when fields do not exist on matching documents")
  def byContainsMatchNoFields(): Unit =
    Using(PgDB()) { db => RemoveFieldsFunctions. byContainsMatchNoFields(db) }

  @Test
  @DisplayName("byContains succeeds when no matching documents exist")
  def byContainsNoMatch(): Unit =
    Using(PgDB()) { db => RemoveFieldsFunctions. byContainsNoMatch(db) }

  @Test
  @DisplayName("byJsonPath removes fields from matching documents")
  def byJsonPathMatchFields(): Unit =
    Using(PgDB()) { db => RemoveFieldsFunctions. byJsonPathMatchFields(db) }

  @Test
  @DisplayName("byJsonPath succeeds when fields do not exist on matching documents")
  def byJsonPathMatchNoFields(): Unit =
    Using(PgDB()) { db => RemoveFieldsFunctions. byJsonPathMatchNoFields(db) }

  @Test
  @DisplayName("byJsonPath succeeds when no matching documents exist")
  def byJsonPathNoMatch(): Unit =
    Using(PgDB()) { db => RemoveFieldsFunctions. byJsonPathNoMatch(db) }
