package solutions.bitbadger.documents.scala.tests.integration

import org.junit.jupiter.api.{DisplayName, Test}

import scala.util.Using

/**
 * SQLite integration tests for the `Document` object / `insert`, `save`, `update` connection extension functions
 */
@DisplayName("Scala | SQLite: Document")
class SQLiteDocumentIT:
  
  @Test
  @DisplayName("insert works with default values")
  def insertDefault(): Unit =
    Using(SQLiteDB()) { db => DocumentFunctions.insertDefault(db) }

  @Test
  @DisplayName("insert fails with duplicate key")
  def insertDupe(): Unit =
    Using(SQLiteDB()) { db => DocumentFunctions.insertDupe(db) }

  @Test
  @DisplayName("insert succeeds with numeric auto IDs")
  def insertNumAutoId(): Unit =
    Using(SQLiteDB()) { db => DocumentFunctions.insertNumAutoId(db) }

  @Test
  @DisplayName("insert succeeds with UUID auto ID")
  def insertUUIDAutoId(): Unit =
    Using(SQLiteDB()) { db => DocumentFunctions.insertUUIDAutoId(db) }

  @Test
  @DisplayName("insert succeeds with random string auto ID")
  def insertStringAutoId(): Unit =
    Using(SQLiteDB()) { db => DocumentFunctions.insertStringAutoId(db) }

  @Test
  @DisplayName("save updates an existing document")
  def saveMatch(): Unit =
    Using(SQLiteDB()) { db => DocumentFunctions.saveMatch(db) }

  @Test
  @DisplayName("save inserts a new document")
  def saveNoMatch(): Unit =
    Using(SQLiteDB()) { db => DocumentFunctions.saveNoMatch(db) }

  @Test
  @DisplayName("update replaces an existing document")
  def updateMatch(): Unit =
    Using(SQLiteDB()) { db => DocumentFunctions.updateMatch(db) }

  @Test
  @DisplayName("update succeeds when no document exists")
  def updateNoMatch(): Unit =
    Using(SQLiteDB()) { db => DocumentFunctions.updateNoMatch(db) }
